/*

* Copyright (c) 2004, The Hong Kong University of Science and Technology (HKUST)
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the university nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/


/*

* Author: Hejun Wu (vmnet@cs.ust.hk)

********************************************************************************/
package net.tinyos.packet;

import java.net.*;
import java.io.*;
import net.tinyos.packet.*;
import net.tinyos.sim.*;
import net.tinyos.sim.event.*;

public class VMNetSource extends AbstractSource {
  private Socket eventSocket = null;
  private Socket cmdSocket = null;
  protected SimProtocol eventProtocol = null;
  protected SimProtocol cmdProtocol = null;
  protected InputStream VMNetEv_is, VMNetCm_is;
  protected OutputStream VMNetEv_os, VMNetCm_os;
  private DataInputStream dis;
  private DataOutputStream  dos; 
  static int MTU_SIZE_UART;
  String kind, host;

  public VMNetSource(String kind, String host) {
    super(kind + "@" + host);
    this.kind = kind;
    this.host = host;
    MTU_SIZE_UART=1024;
  }

  protected void openSource() throws IOException {
    message("Opening " + kind + " source");
    try {
      message("Connecting to VMNet at " + host + ":" +
	      SimProtocol.TOSSIM_EVENT_PORT);
      eventSocket = new Socket(host, SimProtocol.TOSSIM_EVENT_PORT);
      VMNetEv_is = eventSocket.getInputStream();
      VMNetEv_os = eventSocket.getOutputStream();
      dis =new DataInputStream(VMNetEv_is);
   //   eventProtocol = new SimProtocol(VMNetEv_is, VMNetEv_os);
      
      
      
      message("Connecting to VMNet port at" + host + ":" +
	      SimProtocol.TOSSIM_COMMAND_PORT);
      cmdSocket = new Socket(host, SimProtocol.TOSSIM_COMMAND_PORT);
      VMNetCm_is = cmdSocket.getInputStream();
      VMNetCm_os = cmdSocket.getOutputStream();
      //cmdProtocol = new SimProtocol(VMNetCm_is, VMNetCm_os);
      
      dos = new DataOutputStream(new BufferedOutputStream(VMNetCm_os));
      message("Connection opened to VMNet ");
    }
    catch (IOException e) {
      closeSockets();
      throw e;
    }
  }

  protected void closeSockets() throws IOException {
    if (eventSocket != null) {
      eventSocket.close();
    }
    if (cmdSocket != null) {
      cmdSocket.close();
    }
    eventSocket = null;
    cmdSocket = null;
  }

  public void closeSource() throws IOException {
    message("Closing " + kind + " source");
    closeSockets();
  }
public byte readSourceByte() throws IOException {
	return dis.readByte();
}
  public byte[] readSourcePacket() throws IOException {
      byte[] received ;
      for(;;){
      	short datalen = dis.readShort();
	//System.err.println(" heheheheh receiving the bytes he lenth ==  : " + datalen);	
	received =new byte [datalen];
      	dis.readFully(received);
	if(received!=null)
	{
	//VMNet temp debugging...+++++++++++++++++
	/*for(int i=0;i<datalen;i++)
	{
	String bs = Integer.toHexString(received[i] & 0xff).toUpperCase();
	System.err.print(bs);
	System.err.print("   ");
	}*/

	/*if(datalen>37)
	{
	if(received[36]==1)
	System.err.println("1");	
	}*/
	//++++++++++++++++++++++++++++++++++++++
	return received; 
	}
	}
      // Remove extraneous bytes
	     
  }

  public boolean writeSourcePacket(byte[] packet, int len) throws IOException {
   // cmdProtocol.writeCommand(writeVMNetPacket(packet)); 
    dos.write(packet, 0, len);
    dos.flush();
    return true;
  }

}

 
